@@ -72,7 +72,7 @@ public:
 protected:
     bool eventFilter(QObject* obj, QEvent* event) override;
 
-private slots:
+private Q_SLOTS:
     void slotAnimationStateChanged(QAbstractAnimation::State newState,
                                    QAbstractAnimation::State oldState);
 
